@@ -6,6 +6,7 @@
 
 #include <qpdf/PCRE.hh>
 #include <qpdf/QTC.hh>
+#include <qpdf/QPDFExc.hh>
 
 #include <stdexcept>
 #include <string.h>
@@ -15,6 +16,10 @@ static bool is_hex_digit(char ch)
 {
     return (strchr("0123456789abcdefABCDEF", ch) != 0);
 }
+static bool is_space(char ch)
+{
+    return (strchr(" \f\n\r\t\v", ch) != 0);
+}
 
 QPDFTokenizer::QPDFTokenizer() :
     pound_special_in_name(true)
@@ -45,10 +50,90 @@ QPDFTokenizer::reset()
 }
 
 void
-QPDFTokenizer::presentCharacter(char ch)
+QPDFTokenizer::resolveLiteral()
 {
     PCRE num_re("^[\\+\\-]?(?:\\.\\d+|\\d+(?:\\.\\d+)?)$");
 
+    if ((val.length() > 0) && (val[0] == '/'))
+    {
+        type = tt_name;
+        // Deal with # in name token.  Note: '/' by itself is a
+        // valid name, so don't strip leading /.  That way we
+        // don't have to deal with the empty string as a name.
+        std::string nval = "/";
+        char const* valstr = val.c_str() + 1;
+        for (char const* p = valstr; *p; ++p)
+        {
+            if ((*p == '#') && this->pound_special_in_name)
+            {
+                if (p[1] && p[2] &&
+                    is_hex_digit(p[1]) && is_hex_digit(p[2]))
+                {
+                    char num[3];
+                    num[0] = p[1];
+                    num[1] = p[2];
+                    num[2] = '\0';
+                    char ch = (char)(strtol(num, 0, 16));
+                    if (ch == '\0')
+                    {
+                        type = tt_bad;
+                        QTC::TC("qpdf", "QPDF_Tokenizer null in name");
+                        error_message =
+                            "null character not allowed in name token";
+                        nval += "#00";
+                    }
+                    else
+                    {
+                        nval += ch;
+                    }
+                    p += 2;
+                }
+                else
+                {
+                    QTC::TC("qpdf", "QPDF_Tokenizer bad name");
+                    type = tt_bad;
+                    error_message = "invalid name token";
+                    nval += *p;
+                }
+            }
+            else
+            {
+                nval += *p;
+            }
+        }
+        val = nval;
+    }
+    else if (num_re.match(val.c_str()))
+    {
+        if (val.find('.') != std::string::npos)
+        {
+            type = tt_real;
+        }
+        else
+        {
+            type = tt_integer;
+        }
+    }
+    else if ((val == "true") || (val == "false"))
+    {
+        type = tt_bool;
+    }
+    else if (val == "null")
+    {
+        type = tt_null;
+    }
+    else
+    {
+        // I don't really know what it is, so leave it as tt_word.
+        // Lots of cases ($, #, etc.) other than actual words fall
+        // into this category, but that's okay at least for now.
+        type = tt_word;
+    }
+}
+
+void
+QPDFTokenizer::presentCharacter(char ch)
+{
     if (state == st_token_ready)
     {
 	throw std::logic_error(
@@ -175,7 +260,7 @@ QPDFTokenizer::presentCharacter(char ch)
 	    string_ignoring_newline = false;
 	}
 
-	unsigned int bs_num_count = strlen(bs_num_register);
+	size_t bs_num_count = strlen(bs_num_register);
 	bool ch_is_octal = ((ch >= '0') && (ch <= '7'));
 	if ((bs_num_count == 3) || ((bs_num_count > 0) && (! ch_is_octal)))
 	{
@@ -337,81 +422,7 @@ QPDFTokenizer::presentCharacter(char ch)
 
     if ((state == st_token_ready) && (type == tt_word))
     {
-	if ((val.length() > 0) && (val[0] == '/'))
-	{
-	    type = tt_name;
-	    // Deal with # in name token.  Note: '/' by itself is a
-	    // valid name, so don't strip leading /.  That way we
-	    // don't have to deal with the empty string as a name.
-	    std::string nval = "/";
-	    char const* valstr = val.c_str() + 1;
-	    for (char const* p = valstr; *p; ++p)
-	    {
-		if ((*p == '#') && this->pound_special_in_name)
-		{
-		    if (p[1] && p[2] &&
-			is_hex_digit(p[1]) && is_hex_digit(p[2]))
-		    {
-			char num[3];
-			num[0] = p[1];
-			num[1] = p[2];
-			num[2] = '\0';
-			char ch = (char)(strtol(num, 0, 16));
-			if (ch == '\0')
-			{
-			    type = tt_bad;
-			    QTC::TC("qpdf", "QPDF_Tokenizer null in name");
-			    error_message =
-				"null character not allowed in name token";
-			    nval += "#00";
-			}
-			else
-			{
-			    nval += ch;
-			}
-			p += 2;
-		    }
-		    else
-		    {
-			QTC::TC("qpdf", "QPDF_Tokenizer bad name");
-			type = tt_bad;
-			error_message = "invalid name token";
-			nval += *p;
-		    }
-		}
-		else
-		{
-		    nval += *p;
-		}
-	    }
-	    val = nval;
-	}
-	else if (num_re.match(val.c_str()))
-	{
-	    if (val.find('.') != std::string::npos)
-	    {
-		type = tt_real;
-	    }
-	    else
-	    {
-		type = tt_integer;
-	    }
-	}
-	else if ((val == "true") || (val == "false"))
-	{
-	    type = tt_bool;
-	}
-	else if (val == "null")
-	{
-	    type = tt_null;
-	}
-	else
-	{
-	    // I don't really know what it is, so leave it as tt_word.
-	    // Lots of cases ($, #, etc.) other than actual words fall
-	    // into this category, but that's okay at least for now.
-	    type = tt_word;
-	}
+        resolveLiteral();
     }
 
     if (! (betweenTokens() || ((state == st_token_ready) && unread_char)))
@@ -423,22 +434,19 @@ QPDFTokenizer::presentCharacter(char ch)
 void
 QPDFTokenizer::presentEOF()
 {
-    switch (state)
+    if (state == st_literal)
     {
-      case st_token_ready:
-      case st_top:
-	// okay
-	break;
-
-      case st_in_comment:
-	state = st_top;
-	break;
-
-      default:
+        QTC::TC("qpdf", "QPDF_Tokenizer EOF reading appendable token");
+        resolveLiteral();
+    }
+    else if (state != st_token_ready)
+    {
+        QTC::TC("qpdf", "QPDF_Tokenizer EOF reading token");
 	type = tt_bad;
 	error_message = "EOF while reading token";
-	state = st_token_ready;
     }
+
+    state = st_token_ready;
 }
 
 bool
@@ -460,3 +468,55 @@ QPDFTokenizer::betweenTokens()
 {
     return ((state == st_top) || (state == st_in_comment));
 }
+
+QPDFTokenizer::Token
+QPDFTokenizer::readToken(PointerHolder<InputSource> input,
+                         std::string const& context)
+{
+    qpdf_offset_t offset = input->tell();
+    Token token;
+    bool unread_char;
+    char char_to_unread;
+    bool presented_eof = false;
+    while (! getToken(token, unread_char, char_to_unread))
+    {
+	char ch;
+	if (input->read(&ch, 1) == 0)
+	{
+            if (! presented_eof)
+            {
+                presentEOF();
+                presented_eof = true;
+            }
+            else
+            {
+                throw std::logic_error(
+                    "getToken returned false after presenting EOF");
+            }
+	}
+	else
+	{
+	    if (is_space((unsigned char)ch) &&
+		(input->getLastOffset() == offset))
+	    {
+		++offset;
+	    }
+	    presentCharacter(ch);
+	}
+    }
+
+    if (unread_char)
+    {
+	input->unreadCh(char_to_unread);
+    }
+
+    if (token.getType() == tt_bad)
+    {
+	throw QPDFExc(qpdf_e_damaged_pdf, input->getName(),
+		      context, offset, token.getErrorMessage());
+    }
+
+    input->setLastOffset(offset);
+
+    return token;
+}
