@@ -2,7 +2,7 @@
 
 #include <qpdf/QUtil.hh>
 
-QPDF_Integer::QPDF_Integer(int val) :
+QPDF_Integer::QPDF_Integer(long long val) :
     val(val)
 {
 }
@@ -17,7 +17,7 @@ QPDF_Integer::unparse()
     return QUtil::int_to_string(this->val);
 }
 
-int
+long long
 QPDF_Integer::getVal() const
 {
     return this->val;
