@@ -17,7 +17,10 @@ static void report_errors()
 	printf("warning: %s\n", qpdf_get_error_full_text(qpdf, e));
 	printf("  code: %d\n", qpdf_get_error_code(qpdf, e));
 	printf("  file: %s\n", qpdf_get_error_filename(qpdf, e));
-	printf("  pos : %ld\n", qpdf_get_error_file_position(qpdf, e));
+        /* If your compiler doesn't support %lld, change to %ld and
+         * lose precision in the error message.
+         */
+	printf("  pos : %lld\n", qpdf_get_error_file_position(qpdf, e));
 	printf("  text: %s\n", qpdf_get_error_message_detail(qpdf, e));
     }
     if (qpdf_has_error(qpdf))
@@ -27,7 +30,8 @@ static void report_errors()
 	printf("error: %s\n", qpdf_get_error_full_text(qpdf, e));
 	printf("  code: %d\n", qpdf_get_error_code(qpdf, e));
 	printf("  file: %s\n", qpdf_get_error_filename(qpdf, e));
-	printf("  pos : %ld\n", qpdf_get_error_file_position(qpdf, e));
+        /* see above comment about %lld */
+	printf("  pos : %lld\n", qpdf_get_error_file_position(qpdf, e));
 	printf("  text: %s\n", qpdf_get_error_message_detail(qpdf, e));
     }
     else
@@ -346,7 +350,7 @@ static void test16(char const* infile,
     print_info("/Producer");
     print_info("/Creator");
     qpdf_set_info_key(qpdf, "/Author", "Mr. Potato Head");
-    qpdf_set_info_key(qpdf, "/Producer", "QPDF libary");
+    qpdf_set_info_key(qpdf, "/Producer", "QPDF library");
     qpdf_set_info_key(qpdf, "/Creator", 0);
     print_info("/Author");
     print_info("/Producer");
